## pipeline2.cl -- IRC imaging data reduction pipeline step2
##                 based on pipeline.cl created by Y. Ita
## created: F. Egusa, 2010/10/25
## modified: 2010/12/01 (F. Egusa), not to delete the original input files
##                                  set max value for BLANK

procedure pipeline2

string  ircconst 	 {"constants.database", prompt="IRC constants database file name"}
string 	com_mode 	 {"median", prompt="combine mode: average/median"}
int	com_area 	 {2,min=1,max=2, prompt="stack area: 1:common area 2:whole area"}
int	sky_area 	 {2,min=1,max=2, prompt="sky matching area: 1:common area 2:whole area"}
real    det_sig          {3.,min=1.,max=1000., prompt="Threshold in sigma for source detection"}
real    sig_rej          {3.,min=0.,max=100., prompt="Rejection unit in sigmas"}
int     max_itr          {10,min=1,max=10, prompt="Maximum number of iterations"}
string 	obslog 		 {"irclog", prompt="output of the mkirclog"}
bool 	wcs = no         {prompt="Try to compute wcs and write it in the header?"}
bool 	coaddshort = no  {prompt="Coadd short exposure frames?"}
bool 	interactive = no {prompt="Run pipeline interactively?"}
bool 	deltemp = yes    {prompt="Delete intermediate files?"}
string  rejection        {"sigclip", prompt="rej in coadd:minmax|ccdclip|crreject|sigclip|avsigclip|pclip|none"}
bool    rej_sky = yes    {prompt="Remove image(s) with outlier sky level?"}
bool    skip_L = no      {prompt="Skip coadding MIRL images?"}
bool    submedsky = no   {prompt="Subtract median filtered sky in adjust_sky?"}
bool 	coadd   = yes    {prompt="Coadd?"}
bool 	verbose = no     {prompt="Print verbose progress messages?\n"}
string  prefixs          {"fCcDlnw", prompt="Prefix after pipeline1"}
real    blankvalue       {-9999.9,max=-1000., prompt="value for blank/masked pixels"}

string  mode="al"
struct 	*list0

begin
 string irc_const
 string commode
 int comarea
 int skyarea
 real detsig
 real sigrej
 int maxitr
 string logfile
 bool coadd_short, ia, verb, deltmp, setwcs, rejsky, skipl, sub_medsky, coadd_frames, cosmic
 string ans
 string rejtype
 string prefix
 real blank

 print ("\n")
 print ("### PIPELINE step2 ###\n")
 print ("IRC imaging data reduction pipeline start...\n")

# Get query parameter
 irc_const = ircconst
 commode = com_mode
 comarea = com_area
 skyarea = sky_area
 detsig = det_sig
 sigrej = sig_rej
 maxitr = max_itr
 logfile = obslog
 setwcs = wcs
 coadd_short = coaddshort
 ia = interactive
 deltmp = deltemp
 rejtype = rejection
 verb = verbose
 rejsky = rej_sky
 sub_medsky = submedsky
 skipl = skip_L
 coadd_frames=coadd
 prefix = prefixs
 blank = blankval


# make sure that prepipeline has been processed.
 if( !access(logfile) ){
	print ("You should run prepipeline first.")
	bye
 }


## if interactive = yes
 if (ia) print("Pipeline runs in INTERACTIVE mode\n")


## mask hot/bad pixels, slit area, saturated pixels
## add prefix of "m"
## add 'blank' in the header
 if(ia){
        print("\n \n Masking hot/bad pixels, slit area, saturated pixels.\n")
        print("Proceed ? -> y/s/e (Yes/Skip/End), default=y:\n")
        ans = "y" ; scan(ans)
        if(ans=="e"||ans=="E"){
                bye
        }else if(ans=="s"||ans=="S"){
                print("MASKING is skipped.\n") ; prefix = ""//prefix ; goto next01
        }
 }
 ## set deltmp = no to keep the original inputs
 maskall(irc_const, logfile, prefix, blank, no, verb) ; prefix = "m"//prefix
# print("debug: pipeline2.cl l.101")


next01:
## Aspect ratio resampling
## add prefix of "e"
## resampled to 1:1
 if(ia){
	print("\n \nResample aspect ratio.\n")
	print("Proceed ? -> y/e (Yes/End), default=y:\n")
	ans = "y" ; scan(ans)
        if(ans=="e"||ans=="E"){
		bye
        }
 }
# print("debug: pipeline2.cl l.117")
# printf("debug: irc_const=%s, logfile=%s, prefix=%s, blank=%.1f, deltmp=%b, verb=%b\n\n",irc_const,logfile,prefix,blank,deltmp,verb)
 aspect_ratio(irc_const, logfile, prefix, blank, deltmp, verb) ; prefix = "e"//prefix

next02:
## Coadd
 if(ia){
	print("\n \nStack images and put wcs.\n")
	print("Proceed ? -> y/s/e (Yes/Skip/End), default=y:\n")
	ans = "y" ; scan(ans)
        if(ans=="e"||ans=="E"){
		goto finish
        }else if(ans=="s"||ans=="S"){
		print("Image stacking is skipped.\n") ; goto finish
	}else{
		coadd_frames = yes
	}
 }
 if(coadd_frames){
	 coadd(irc_const, detsig, sigrej, maxitr, commode, comarea, skyarea, logfile, prefix, rejtype, rejsky, coadd_short, setwcs, skipl, sub_medsky, deltmp, verb,blank)
 }


finish:
## Finish
 print ("\n")
 print ("### PIPELINE step2 finished!! ###\n")
 print ("\n\n")


end
